# copyright 2017 Yi Wang
# don't distribute without author's explicit authorization

from distutils.core import setup

setup(
    name='badminton_queuing_system',
    packages=['badminton_queuing_system'],
    version='0.12',
    description='Badminton Queuing system tailored for CBA',
    url='http://github.com/yiw2000/badminton_queuing_system',
    author='Yi Wang',
    author_email='yiw2000@gmail.com',
)